@@ -6,16 +6,12 @@ _spacy_available = importlib.util.find_spec("spacy") is not None
 
 
 def is_torch_available():
-    """
-    Check if PyTorch is available.
-    """
+    """Check if PyTorch is available."""
     return _torch_available
 
 
 def is_spacy_available():
-    """
-    Check if spaCy is available.
-    """
+    """Check if spaCy is available."""
     return _spacy_available
 
 
@@ -27,7 +23,11 @@ if is_torch_available():
 def get_logger(name: str) -> logging.Logger:
     """
     Return the logger of the given name.
-    :param name: name of the logger to return
-    :return: the logger
+
+    Args:
+        name (`str`): The name of the logger.
+
+    Returns:
+        :obj:`logging.Logger`: The logger of the given name.
     """
     return logging.getLogger(name)
